{   -    ,
  Unit1.pas}
unit MyFunc;

interface

uses SysUtils, Windows, ComObj, Variants, Classes;

  function FirstWord(const S: string; Sep: Char = ' '): string;
  function FirstSimpleAfter(m: integer): integer;
  function PJWHash(const aKey: string; aTableSize: integer): integer;
  function PosCh(aCh: Char; const S: string; StartPos: integer = 1): integer;
  function PosN(const P: string; const S: string; StartPos: integer = 1): integer;
  function IsDos(const S: string): boolean;
  function IsKoi(const S: string):boolean;
  function IsUTF(const S: string): boolean;
  function Dos2Win(const aStr: String): String;
  function KOI8ToWin(const aStr: String): String;
  function UTF8ToStr(Value: String): String;
  function UTF8ToStrSmart(Value: String): String;
  function KillTags(S: string): string;
  function IsBB(const w: string): boolean;
  function IsBadAbb(const w: string; m: integer): boolean;
  function Body(const W: string): string;
  function Tail(const W: string): string;
  function FilterByMask(S, Mask, Dest: string): string;
  function MatchStrings(source, pattern: string):Boolean;
  function Doc2Txt(DocName: string): string;
  function Similarity(const X, Y: string; M: integer): real;
  //function Doc2Rtf(DocName: string): string;
  function SAPI5cbX(): string;
  function SAPI4cbX(): string;
  function Puff(const s: string; MaxLength: integer): string;
  procedure Fragments(const S: string; FS: integer; var Frag, FragPos: TStringList);
  procedure SplitStr(const S: string; var Words: TStringList; Sep: Char = ' ');
  procedure SplitText(const S: string; var Words: TStringList);

implementation

{   }
function FirstWord(const S: string; Sep: Char = ' '): string;
var
  k: integer;
begin
  k := PosCh(Sep,S);
  if k > 0 then Result := Copy(S,1,k-1)
           else Result := S;
end;


{  ,    m}
function FirstSimpleAfter(m: integer): integer;
var
  n, k: integer;
  simple: boolean;
begin
  n := m;
  if n mod 2 = 0 then Inc(n);
  while True do begin
    k := 3;
    simple := True;
    while k*k<n do begin
      if (n mod k = 0) then simple := False;
      Inc(k);
    end;
    if not simple then Inc(n,2)
                  else begin
                    Result := n;
                    exit;
                  end;
  end;
end;


{-(P.J.Weinberger).  , 
aTableSize   }
function PJWHash(const aKey: string; aTableSize: integer): integer;
var
  G: longint;
  i: integer;
  Hash: longint;
begin
  Hash := 0;
  for i := 1 to Length(akey) do begin
    Hash := (Hash shl 4)+ord(aKey[i]);
    G := Hash and longint($F0000000);
    if (G <> 0) then
      Hash := (Hash xor (G shr 24)) xor G;
  end;
  Result := Hash mod aTableSize;
end;


{   .   5    Pos(), ..
       }
function PosCh(aCh: Char; const S: string; StartPos: integer = 1): integer;
var
  i: integer;
begin
  Result := 0;
  for i := StartPos to Length(S) do
    if (S[i] = aCh) then begin
      Result := i;
      exit;
    end;
end;


{     " ".    
 Pos(), ..      ,   
 }
function PosN(const P: string; const S: string; StartPos: integer = 1): integer;
var
  i, j: Integer;
begin
  Result := 0;
  if Length(P) > Length(S) then exit;
  for i := StartPos to Length(S) - Length(P) + 1 do
    for j := 1 to Length(P) do
      if P[j] <> S[i + j - 1] then break
      else if j = Length(P) then begin
        Result := i;
        exit;
      end;
end;


{   ,   ,   }
function Probe(const S: string; m: integer): string;
var
  S1, S2, S3: string;
  n: integer;
begin
  n := Length(S);
  if n <= 4*m then begin
    Result := S;
    exit;
  end;
  S1 := Copy(S,1,m);
  S2 := Copy(S,Round(n/2)-m,2*m);
  S3 := Copy(S,n-m,m);
  Result := S1+' '+S2+' '+S3;
end;


{  DOS      }
function IsDos(const S: string): boolean;
  var
    i, win, dos: integer;
    Frag: string;
  begin
    win := 0; dos := 0; Result := False;
    if S = '' then exit;
    Frag := Probe(Trim(S),80);
    for i := 1 to Length(Frag) do begin
      case ord(Frag[i]) of
        128..175: Inc(dos);
        196..255: Inc(win);
      end;
    end;
    if win < dos then Result:=true;
end;


function Dos2Win(const aStr: String): String;
begin
  Result := aStr;
  if Result='' then Exit;
  OemToChar(PChar(Result),PChar(Result));
end;


function IsKoi(const S: string):boolean;
  {  KOI8-R      
  ''  ''.  KOI8-R    ''  ''.  
   ""   33   "",  ""  3.3   "".
    ""     "", ...    :).
      250    100%-}
var
    i, win, koi: integer;
    Frag: string;
begin
  win := 0; koi := 0; Result := False;
  if S = '' then exit;
  Frag := Probe(Trim(S),400);
  for i := 1 to Length(Frag) do begin
    case ord(Frag[i]) of
      207: Inc(koi);
      239: Inc(koi);
      192: Inc(win);
      224: Inc(win);
    end;
  end;
  //if win < koi then result := True;
  {  ,     .
    win-   1  :)}
  if win+1 < koi then result := True;
end;


function IsUTF(const S: string): boolean;
var
  c: string;
begin
  Result := False;
  c := AnsiLowerCase(Copy(S,1,512));
  if (PosN('encoding="utf-8"',S) > 0) or (PosN('charset=utf-8"',S) > 0) then
    Result := True;
end;


function KOI8ToWin(const aStr: String): String;
var
  n,i,j,k: integer;
  const
  Koi: string =
  #128#129#130#131#132#133#134#135#136#137#138#139#140#141#142#143+
  #144#145#146#147#148#149#150#151#152#153#154#155#156#157#158#159+
  #160#161#162#163#164#165#166#167#168#169#170#171#172#173#174#175+
  #176#177#178#179#180#181#182#183#184#185#186#187#188#189#190#191+
  #192#193#194#195#196#197#198#199#200#201#202#203#204#205#206#207+
  #208#209#210#211#212#213#214#215#216#217#218#219#220#221#222#223+
  #224#225#226#227#228#229#230#231#232#233#234#235#236#237#238#239+
  #240#241#242#243#244#245#246#247#248#249#250#251#252#253#254#255;
  Win: string =
  #128#129#130#131#132#133#134#135#136#137#138#139#140#141#142#143+
  #144#145#146#147#148#149#150#151#152#153#218#155#176#157#183#159+
  #160#161#162#184#186#165#166#191#168#169#170#171#172#173#174#175+
  #156#177#178#168#170#181#182#175#184#185#186#187#188#189#190#185+
  #254#224#225#246#228#229#244#227#245#232#233#234#235#236#237#238+
  #239#255#240#241#242#243#230#226#252#251#231#248#253#249#247#250+
  #222#192#193#214#196#197#212#195#213#200#201#202#203#204#205#206+
  #207#223#208#209#210#211#198#194#220#219#199#216#221#217#215#218;
begin
  Result := '';
  if aStr = '' then Exit;
  n := Length(aStr);
  for i:=1 to n do begin
    k := 0;
    for j := 1 to 128 do
      if aStr[i] = Koi[j] then begin
        k := j;
        break;
      end;
    if k <> 0 then Result := Result+Win[k]
              else Result := Result+aStr[i];
  end;
end;

{ 2    ,  - .
     
- UTF8ToAnsi(),    , 
    }

{Convert string from UTF-8 format into ASCII}
function UTF8ToStr(Value: String): String;
var
  buffer: Pointer;
  BufLen: LongWord;
begin
  BufLen := Length(Value) + 4;
  GetMem(buffer, BufLen);
  FillChar(buffer^, BufLen, 0);
  MultiByteToWideChar(CP_UTF8, 0, @Value[1], BufLen - 4, buffer, BufLen);
  Result := WideCharToString(buffer);
  FreeMem(buffer, BufLen);
end;

{Convert string from UTF-8 format mixed with standart ASCII symbols($00..$7f)}
function UTF8ToStrSmart(Value: String): String;
var
  Digit: String;
  i: integer; // Word;
  HByte: Byte;
  Len: Byte;
begin
  Result := '';
  Len := 0;
  if Value = '' then Exit;
  for i := 1 to Length(Value) do
  begin
    if Len > 0 then
    begin
      Digit := Digit + Value[i];
      Dec(Len);
      if Len = 0 then
        Result := Result + UTF8ToStr(Digit);
    end else
    begin
      HByte := Ord(Value[i]);
      if HByte in [$00..$7f] then       //Standart ASCII chars
        Result := Result + Value[i]
      else begin
        //Get length of UTF-8 char
        if HByte and $FC = $FC then
          Len := 6
        else if HByte and $F8 = $F8 then
          Len := 5
        else if HByte and $F0 = $F0 then
          Len := 4
        else if HByte and $E0 = $E0 then
          Len := 3
        else if HByte and $C0 = $C0 then
          Len := 2
        else begin
          Result := Result + Value[i];
          Continue;
        end;
        Dec(Len);
        Digit := Value[i];
      end;
    end;
  end;
end; 


{   html- }
function KillTags(S: string): string;
const
  a: array[0..5] of string = ('&copy;','&nbsp;','&amp;','&lt;','&gt;','&quot;');
  b: array[0..5] of string = ('(c)',' ','&','<','>','"');
var
  i, j: integer;
  htag: boolean;
  copied, c: string;
begin
  htag := False;
  Result := '';
  i := 0;
  while i <= Length(S) do begin
    Inc(i);
    if (S[i] = '&') and not htag then begin
      copied := Copy(S,i,6);
      for j := 0 to high(a) do
        if copy(copied,1,Length(a[j])) = a[j] then begin
          Result := Result+b[j];
          i := i+Length(a[j])-1;
        end;
    end
    else begin
      if S[i] = '<' then htag := True;
      c := S[i];
      if c = #150 then c := '-';   {: }
      if c = #151 then c := '-';   {  }
      if c = #160 then c := ' ';   {"" ( !!!)}
      if c = #133 then c := '...'; { }
      if (c = #171) or (c = #187) then c := '"';  {"" }
      if (c = #132) or (c = #147) then c := '"';  {/ }
      if not htag then Result := Result+c;
      if htag and (i < Length(S)-1) and (AnsiLowerCase(Copy(S,i,3)) = '<p>')
        then Result := Result+#13#10;
      if S[i] = '>' then htag := False;
    end;
  end;
end;


{   1-     )}
function IsBB(const w: string): boolean;
begin
  Result := False;
  if w = '' then exit;
  if w[1] in [''..'','','A'..'Z'] then begin
    Result := True;
    exit;
  end;
  if (Length(w) > 1) and (w[1] = '"') and (w[2] in [''..'','','A'..'Z'])
    then begin
      Result := True;
      exit;
    end;
end;


{      -   ,
  ,   !}
function IsBadAbb(const w: string; m: integer): boolean;
var
  i, k, n: integer;
begin
  Result := False;
  if w = '' then exit;
  n := Length(w);
  k := 0;
  for i := 1 to n do if w[i] in [''..'','','A'..'Z'] then begin
    Inc(k);
    if (k > m) or (w[i] in ['','','','','']) then begin
      Result := True;
      exit;
    end;
  end;
end;

{   X, Y. 0 -  , 1 - ,  .
    X,   M ,   Y  
      .  "" 
! Sim(X,Y)  ,  , Sim(Y,X).   
:  X    Y,  Sim(X,Y) = 1}
function Similarity(const X, Y: string; M: integer): real;
function Sim(const X, Y: string; M: integer): real;
var
  n, i, j, k, Good, Vsego: integer;
  a, b, c: string;
  R: real;
begin
  R := 0;
  n := Length(X);
  Vsego := 0;
  Good := 0;
  a := X;
  b := Y;
  for i := n-m to n-1 do
    for j := 1 to i+1 do begin
      c := Copy(a,j,n-i);
      Inc(Vsego);
      if Pos(AnsiLowerCase(c),AnsiLowerCase(b)) > 0 then Inc(Good);
    end;
  if Vsego > 0 then R := Good/Vsego;
  Result := R;
end;  {Sim}
var
  a, b: real;
begin
 {    }
  a := Sim(X,Y,M);
  b := Sim(Y,X,M);
  if a > b then Similarity := a
           else Similarity := b;
end;


{   " "  .
Body(', !..') ==> ', '}
function Body(const W: string): string;
var
  N, i: integer;
  a: string;
begin
  N := Length(W);
  a := W;
  for i := N downto 1 do begin
    {   ""}
    if not (W[Ord(i)] in ['.',',',':',';','!','?',#133])
      then begin
         a := Copy(W,1,i);
         break;
      end
  end;
  Result := a;
end;


{   " "  .
Tail(', !..') ==> '!..'}
function Tail(const W: string): string;
var
  N, i: integer;
  b: string;
begin
  N := Length(W);
  b := '';
  for i := N downto 1 do begin
    {   ""}
    if not (W[Ord(i)] in ['.',',',':',';','!','?',#133])
      then begin
         b := Copy(W,i+1,N);
         break;
      end
  end;
  Result := b;
end;


{   S   Mask=Dest. :
               
-------------------------------------------------
=<    =<
*=<          ==> <
                       ==> <
*=               ==> 
                       ==> 
**=        ==> 
-------------------------------------------------
,    "" 
    }
function FilterByMask(S, Mask, Dest: string): string;
begin
  {        $*...=Dest
     $   , ..  Dest  }
  if (Length(S) > 1) and (Length(Mask) > 1)
                     and (Mask[1] = '$') and (Mask[2] = '*') then begin
    S := Copy(S,2,Length(S)-1);
    Mask := Copy(Mask,2,Length(Mask)-1);
  end;
  Result := S;
  {     }
  if S = Mask then begin
    Result := Dest;
    exit;
  end;
  {      }
  if Body(S) = Mask then begin
    Result := Dest+Tail(S);
    exit;
  end;
  { (   !)       }
  if (Mask[1] = '*') and (Mask[Length(Mask)] <> '*') then begin
    if Tail(Mask) = '' {         S}
      then Result := Copy(S,1,Length(S)-Length(Tail(S))-Length(Mask)+1)+Body(Dest)+Tail(S)
      else Result := Copy(S,1,Length(S)-Length(Mask)+1)+Dest;
    exit;
  end;
  { (..   )       }
  if (Mask[1] <> '*') and (Mask[Length(Mask)] = '*') then begin
    Result := Dest+Copy(S,Length(Mask),Length(S)-Length(Mask)+1);
    exit;
  end;
  {        }
  if (Mask[1] = '*') and (Mask[Length(Mask)] = '*') then begin
    Result := StringReplace(S,Copy(Mask,2,Length(Mask)-2),Dest,[rfReplaceAll]);
    exit;
  end;
end;


{*********************************************************************
   .     *  ?.
 ( )    .
: MatchStrings('David Stidolph','*St*') ==> True.
  C-: Sean Stanley.
   Delphi: David Stidolph.
   !
**********************************************************************}
{  !  "?"    !!!
 IsWildPattern -    ,  , 
     :   ,
 pattern -   , ..  * / ?}
Function MatchStrings(source, pattern: string):Boolean;
var
  pSource : Array [0..255] of Char;
  pPattern : Array [0..255] of Char;

Function MatchPattern(element, pattern:PChar): Boolean;
//Function IsPatternWild(pattern: PChar):Boolean;
//Begin
//  Result:=StrScan(pattern,'*')<> nil;
//  If not Result Then Result:=StrScan(pattern,'?')<> nil;
//End;
Begin
  If StrComp(pattern,'*')=0
  Then Result := True
  Else If (element^ = Chr(0)) and (pattern^ <> Chr(0))
       Then Result:=False
       Else If element^ = Chr(0)
            Then Result := True
            Else Begin
              Case pattern^ of
                '*': If MatchPattern(element,@pattern[1])
                     Then Result:=True
                     Else Result := MatchPattern(@element[1],pattern);
              //  '?': Result := MatchPattern(@element[1],@pattern[1]);
              Else If element^=pattern^
                   Then Result := MatchPattern(@element[1],@pattern[1])
                   Else Result := False;
              End;
            End;
End;

Begin
  StrPCopy(pSource,source);
  StrPCopy(pPattern,pattern);
  Result:=MatchPattern(pSource,pPattern);
End;
{**********************************************************************}


{ DOC => TXT   OLE- MS Word.
     txt-    .
!!     : "Undeclared identifier"
 FileName  FileFormat -    :)
  FileFormat:
   0         ( Word)
   1         
   2          (extended characters saved in ANSI character set) 
   3        + (plain text with line breaks; extended 
            characters saved in ANSI character set) 
   4          (PC-8) (extended characters saved in IBM PC 
            character set) 
   5        + (PC-8) (text with line breaks; extended 
            characters saved in IBM PC character set) 
   6         Rich-text (RTF) 
}
function Doc2Txt(DocName: string): string;
var
  WordApp: Variant;
  TxtName: string;
begin
  try
    WordApp := CreateOleObject('Word.Application');
  except
    Result := '';
    exit;
  end;
  TxtName := ChangeFileExt(DocName, '.txt');
  WordApp.Documents.Open(DocName);
  WordApp.ActiveDocument.SaveAs(FileName := TxtName, FileFormat := 2);
  WordApp.ActiveDocument.Close;
  WordApp.Quit;
  WordApp := UnAssigned;
  Result := TxtName;
end;

{. DOC => RTF.   }
//function Doc2Rtf(DocName: string): string;
//var
//  WordApp: Variant;
//  TxtName: string;
//begin
//  try
//    WordApp := CreateOleObject('Word.Application');
//  except
//    Result := '';
//    exit;
//  end;
//  TxtName := ChangeFileExt(DocName, '.rtf');
//  WordApp.Documents.Open(DocName);
//  WordApp.ActiveDocument.SaveAs(FileName := TxtName, FileFormat := 6);
//  WordApp.ActiveDocument.Close;
//  WordApp.Quit;
//  WordApp := UnAssigned;
//  Result := TxtName;
//end;

function SAPI4cbX(): string;
begin
  Result :=  'x1'+#13#10+'x2'+#13#10+'x4'+#13#10+'x8'+#13#10+'x16'+#13#10+
             'x32'+#13#10+'x64'#13#10+'MAX';
end;


function SAPI5cbX(): string;
begin
  Result :=
  'SAFT8kHz8BitMono'    +#13#10+
  'SAFT8kHz8BitStereo'  +#13#10+
  'SAFT8kHz16BitMono'   +#13#10+
  'SAFT8kHz16BitStereo' +#13#10+
  'SAFT11kHz8BitMono'   +#13#10+
  'SAFT11kHz8BitStereo' +#13#10+
  'SAFT11kHz16BitMono'  +#13#10+
  'SAFT11kHz16BitStereo'+#13#10+
  'SAFT12kHz8BitMono'   +#13#10+
  'SAFT12kHz8BitStereo' +#13#10+
  'SAFT12kHz16BitMono'  +#13#10+
  'SAFT12kHz16BitStereo'+#13#10+
  'SAFT16kHz8BitMono'   +#13#10+
  'SAFT16kHz8BitStereo' +#13#10+
  'SAFT16kHz16BitMono'  +#13#10+
  'SAFT16kHz16BitStereo'+#13#10+
  'SAFT22kHz8BitMono'   +#13#10+
  'SAFT22kHz8BitStereo' +#13#10+
  'SAFT22kHz16BitMono'  +#13#10+
  'SAFT22kHz16BitStereo'+#13#10+
  'SAFT24kHz8BitMono'   +#13#10+
  'SAFT24kHz8BitStereo' +#13#10+
  'SAFT24kHz16BitMono'  +#13#10+
  'SAFT24kHz16BitStereo'+#13#10+
  'SAFT32kHz8BitMono'   +#13#10+
  'SAFT32kHz8BitStereo' +#13#10+
  'SAFT32kHz16BitMono'  +#13#10+
  'SAFT32kHz16BitStereo'+#13#10+
  'SAFT44kHz8BitMono'   +#13#10+
  'SAFT44kHz8BitStereo' +#13#10+
  'SAFT44kHz16BitMono'  +#13#10+
  'SAFT44kHz16BitStereo'+#13#10+
  'SAFT48kHz8BitMono'   +#13#10+
  'SAFT48kHz8BitStereo' +#13#10+
  'SAFT48kHz16BitMono'  +#13#10+
  'SAFT48kHz16BitStereo';
end;


{ ,   , ,    
  .    -  , 
 .     }
function Puff(const s: string; MaxLength: integer): string;
var
  w: TStringList;
  m, i, j, n, delta: integer;
begin
  Result := '';
  w := TStringList.Create;
  SplitStr(s,w);
  m := 0;
  n := w.Count;
  if n = 0 then exit;
  for i := 0 to n-1 do begin
    if i < n-1 then w[i] := w[i]+' ';
    m := m+Length(w[i]);
  end;
  delta := MaxLength-m;
  while delta > 0 do begin
    j := Random(n-1);
    if w[j] <> '- ' then begin
      w[j] := w[j]+' ';
      Dec(delta);
    end;
  end;
  for i := 0 to n-1 do Result := Result+w[i];
  w.Free;
end;


{   TStringList      .
    .  ,     -
   .  ,       , 
 (  ) ""     :(}
procedure Fragments(const S: string; FS: integer; var Frag, FragPos: TStringList);
function DelimPos(const S: string; StartPos: integer = 1): integer;
var
  i, j: integer;
  P, Q: string;
begin
  P := #13#10+' ';  {  }
  Q := #13#10#13;   {  - 2 EOL ;     3 }
  Result := 0;
  if Length(S) <3 then exit;
  for i := StartPos to Length(S) - 2 do
    for j := 1 to 3 do
      if (P[j] <> S[i + j - 1]) and (Q[j] <> S[i + j - 1]) then break
      else if j = 3 then begin
        Result := i;
        exit;
      end;
end;
var
  k, k0, N, j, p, FragSize: integer;
  L, Buf: string;
begin
  if FS > 0 then FragSize := FS else FragSize := 999999999;
  Frag.Clear; FragPos.Clear;
  Buf := '';
  p := 1;
  j := 0;  {       EOL}
  N := Length(S);
  k0 := 1;
  k := 1;
  while k > 0 do begin
    //k := PosN(#13#10+' ',S,k);
    k := DelimPos(S,k);  {     }
    if k > 0 then begin  { }
      L := Copy(S,k0,k-k0);
      Buf := Buf+L+#13#10; Inc(j,Length(L)+2);
      if (j > FragSize-Length(L)-2) then begin  { }
        Frag.Add(Buf);  FragPos.Add(IntToStr(p)); j := 0; Inc(p,Length(Buf)); Buf := '';
      end;
      Inc(k,2);
      k0 := k;
    end;
  end;
  if k0 < N then begin
    L := Copy(S,k0,N-k0+1);  { }
    Buf := Buf+L+#13#10; Inc(j,Length(L)+2);
    if (j > FragSize-Length(L)-2) then begin
      Frag.Add(Buf); FragPos.Add(IntToStr(p)); j := 0; Inc(p,Length(Buf)); Buf := '';
    end;
    Inc(k,2);
    k0 := k;
  end;
  if Buf <> '' then begin  {  () }
    Frag.Add(Buf);  FragPos.Add(IntToStr(p)); j := 0; Inc(p,Length(Buf)); Buf := '';
  end;
end;


{ Words    S,   Eol.
     }
procedure SplitStr(const S: string; var Words: TStringList; Sep: Char = ' ');
var
  n, k: integer;
  W: string;
begin
  Words.Clear;
  n := 1;
  k := 1;
  while k >0 do begin
    k := PosCh(Sep,S,k);
    if k >0 then begin
      W := Copy(S,n,k-n);
      Inc(k);
      n := k;
      {    "" !}
      if (W <> '') and (Length(W) < 100) then Words.Add(W);
    end;
  end;
  if k < Length(S) then begin  { "" -  }
    W := Copy(S,n,Length(S)-k+1);
    if (W <> '') and (Length(W) < 100) then Words.Add(W);
  end;
end;


{ Words    ,  Eol.   
  .       
""}
procedure SplitText(const S: string; var Words: TStringList);
var
  WL: TStringList;
  k, k0, N, j: integer;
  L: string;
begin
  Words.Clear;
  WL := TStringList.Create;
  N := Length(S);
  k0 := 1;
  k := 1;
  while k > 0 do begin
    k := PosN(#13#10,S,k);
    if k > 0 then begin  {   }
      L := Copy(S,k0,k-k0);
      if Trim(L) = '' then Words.Add('_E_')
                      else if L[1] = ' ' then Words.Add('_Z_');
      WL.Clear;
      SplitStr(L,WL);
      for j := 0 to WL.Count-1 do Words.Add(WL[j]);
      Inc(k,2);
      k0 := k;
    end;
  end;
  if k0 < N then begin
    L := Copy(S,k0,N-k0+1);  { }
    if Trim(L) = '' then Words.Add('_E_')
    else if L[1] = ' ' then Words.Add('_Z_');
    WL.Clear;
    SplitStr(L,WL);
    for j := 0 to WL.Count-1 do Words.Add(WL[j]);
  end;
  WL.Free;
end;



end.
